unit KOLTitleMenu;

interface

uses
  Windows,
  KOL,
  Messages;

type
{ Windows.pas's COLOR16 declaration is wrong; It should be a DWord instead of ShortInt (This is official) }
  PTitleMenu = ^TTitleMenu;
  TTitleMenu = object(TObj)
    protected
      fMenu: PMENU;
      fFont: PGraphicTool;
      function GradientFill(DC : hDC; pVertex : Pointer; dwNumVertex : DWORD;
        pMesh : Pointer; dwNumMesh, dwMode: DWORD) : Bool; stdcall;
    public
      clLeft, clRight, clText: DWord;
      bDrawEdge: Boolean;
      EdgeStyle: Cardinal;
      procedure ManifestAsTitle( Menu: PMENU; MenuIndex: Integer );
      procedure UnManifestTitle( MenuIndex: Integer );
      function DrawTitle(DC: HDC; const Rect: TRect;
        ItemIdx: Integer; DrawAction: TDrawAction;
        ItemState: TDrawState): Boolean;
      function MeasureTitle(Idx: Integer): Integer;
  end;

  TRIVERTEX = packed record
    X, Y : DWORD;
    Red, Green, Blue, Alpha : Word;
  end;

  functGradientFill = function(DC : hDC; pVertex : Pointer; dwNumVertex : DWORD;
    pMesh : Pointer; dwNumMesh, dwMode: DWORD) : Bool; stdcall;

function NewTitleMenu( DrawEdge: Boolean = False; FontHeight: Integer = 8 ): PTitleMenu;

var
  hinst_msimg32: HInst;
  m_bCanDoGradientFill: Boolean;
  dllfunc_GradientFill: Pointer;

implementation

function NewTitleMenu( DrawEdge: Boolean = False; FontHeight: Integer = 8 ): PTitleMenu;
begin
  New( Result, Create );
  Result.clLeft   := GetSysColor(COLOR_ACTIVECAPTION);
  Result.clRight  := GetSysColor(COLOR_GRADIENTACTIVECAPTION);
  Result.clText   := GetSysColor(COLOR_CAPTIONTEXT);
  Result.fFont    := NewFont;
  Result.fFont.FontHeight := FontHeight;
  Result.fFont.FontStyle  := [fsBold];
  Result.bDrawEdge := DrawEdge;
  Result.EdgeStyle := BDR_SUNKENINNER;
end;

function TTitleMenu.GradientFill(DC : hDC; pVertex : Pointer; dwNumVertex : DWORD;
  pMesh : Pointer; dwNumMesh, dwMode: DWORD) : Bool; stdcall;
begin
  Result := False;
  if m_bCanDoGradientFill then
    Result := functGradientFill(dllfunc_GradientFill)
      ( DC, pVertex, dwNumVertex, pMesh, dwNumMesh, dwMode );
end;

function TTitleMenu.DrawTitle(DC: HDC; const Rect: TRect;
  ItemIdx: Integer; DrawAction: TDrawAction;
  ItemState: TDrawState): Boolean;
var
  pVertex: Array[0..1] of TRIVERTEX;
  pMesh: GRADIENT_RECT;
  crOldBk, crOld: COLORREF;
  modeOld: Integer;
  hfontOld: HFONT;
  ItemRect: TRect;
begin
  crOldBk := SetBkColor( DC, clLeft );
  ItemRect := Rect;

  pVertex[0].x      := Rect.Left;
  pVertex[0].y      := Rect.Top;
  pVertex[0].Red    := GetRValue(clLeft) shl 8;
  pVertex[0].Green  := GetGValue(clLeft) shl 8;
  pVertex[0].Blue   := GetBValue(clLeft) shl 8;
  pVertex[0].Alpha  := $0000;

  pVertex[1].x      := Rect.Right;
  pVertex[1].y      := Rect.Bottom;
  pVertex[1].Red    := GetRValue(clRight) shl 8;
  pVertex[1].Green  := GetGValue(clRight) shl 8;
  pVertex[1].Blue   := GetBValue(clRight) shl 8;
  pVertex[1].Alpha  := $0000;

  pMesh.UpperLeft   := 0;
  pMesh.LowerRight  := 1;
  
  if not ( GradientFill( DC, @pVertex, 2, @pMesh, 1,
             GRADIENT_FILL_RECT_H ) )
  then
    ExtTextOut( DC, 0, 0, ETO_OPAQUE, @ItemRect, nil, 0, nil );

  if bDrawEdge then
    DrawEdge( DC, ItemRect, EdgeStyle, BF_RECT);

  modeOld := SetBkMode( DC, TRANSPARENT );
  crOld := SetTextColor( DC, clText );
  hfontOld := SelectObject( DC, fFont.Handle );
  Inc( ItemRect.Left, GetSystemMetrics(SM_CXMENUCHECK) + 8 );
  DrawText( DC, PChar(fMenu.ItemText[ItemIdx]), -1, ItemRect,
    DT_SINGLELINE or DT_VCENTER or DT_LEFT);

  SelectObject( DC, hfontOld );
  SetBkMode( DC, modeOld );
  SetBkColor( DC, crOldBk );
  SetTextColor( DC, crOld );

  Result := True;
end;

function TTitleMenu.MeasureTitle(Idx: Integer): Integer;
const
  nBorderSize: Integer = 2;
var
  screenDC, memDC: HDC;
  hfontOld: HFONT;
  Size: TSize;
begin
  screenDC:=GetDC( 0 );
  memDC:=CreateCompatibleDC(screenDC);

  hfontOld := SelectObject( memDC, fFont.Handle );
  GetTextExtentPoint32( memDC, PChar(fMenu.ItemText[Idx]),
        Length(fMenu.ItemText[Idx]), Size );
  SelectObject( memDC, hfontOld );
  DeleteDC( memDC );
  ReleaseDC( 0, screenDC );

  Inc( Size.cx, GetSystemMetrics(SM_CXMENUCHECK) + 8 );
  Inc( Size.cx, nBorderSize );
  Inc( Size.cy, nBorderSize );

  Result := (Size.cy and $00FF) or (Size.cx shl 16);
end;

procedure TTitleMenu.ManifestAsTitle( Menu: PMENU; MenuIndex: Integer );
var MenuItemInfo: TMENUITEMINFOA;
begin
  fMenu := Menu;
  Menu.ItemEnabled[MenuIndex] := False;
  // Set ownerdraw
  MenuItemInfo.cbSize := SizeOf(MenuItemInfo);
  MenuItemInfo.fMask := MIIM_FTYPE; // MIM_FTYPE doesn't work under Windows 95
  GetMenuItemInfo(fMenu.Handle, MenuIndex, True, MenuItemInfo);
  MenuItemInfo.fType := MenuItemInfo.fType or MFT_OWNERDRAW;
  SetMenuItemInfo(fMenu.Handle, MenuIndex, True, MenuItemInfo);
end;

procedure TTitleMenu.UnManifestTitle( MenuIndex: Integer );
var MenuItemInfo: TMENUITEMINFOA;
begin
  MenuItemInfo.cbSize := SizeOf(MenuItemInfo);
  MenuItemInfo.fMask := MIIM_FTYPE; // MIM_FTYPE doesn't work under Windows 95
  GetMenuItemInfo(fMenu.Handle, MenuIndex, True, MenuItemInfo);
  MenuItemInfo.fType := MenuItemInfo.fType and not MFT_OWNERDRAW;
  SetMenuItemInfo(fMenu.Handle, MenuIndex, True, MenuItemInfo);
  fMenu.ItemEnabled[MenuIndex] := True;
end;

initialization
  hinst_msimg32 := LoadLibrary( msimg32 );
  if hinst_msimg32 <> 0 then
  begin
    m_bCanDoGradientFill := True;
    dllfunc_GradientFill := GetProcAddress( hinst_msimg32, 'GradientFill' );
  end else
    m_bCanDoGradientFill := False;
    
finalization
  if hinst_msimg32 <> 0 then
    FreeLibrary( hinst_msimg32 );
    
end.
